/*
 *  Copyright (C) 2003 Philip Langdale
 *
 *  Based on egg-radio-action from libegg , which does not have copyright header.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-radio-action.h"
#include "eggradiotoolbutton.h"
#include "gul-gobject-misc.h"

static void	galeon_radio_action_init	(GaleonRadioAction *action);
static void	galeon_radio_action_class_init	(GaleonRadioActionClass *class);
static void	galeon_radio_action_finalize	(GObject *object);
static void	galeon_radio_action_activate	(EggAction *action);

MAKE_GET_TYPE(galeon_radio_action, "GaleonRadioAction",
	      GaleonRadioAction, galeon_radio_action_class_init, 
	      galeon_radio_action_init, GALEON_TYPE_TOGGLE_ACTION);

static GObjectClass *parent_class = NULL;

static void
galeon_radio_action_class_init(GaleonRadioActionClass *class)
{
	GObjectClass *object_class;
	EggActionClass *action_class;

	parent_class = g_type_class_peek_parent(class);
	object_class = G_OBJECT_CLASS(class);
	action_class = EGG_ACTION_CLASS(class);

	action_class->menu_item_type = GTK_TYPE_RADIO_MENU_ITEM;
	action_class->toolbar_item_type = EGG_TYPE_RADIO_TOOL_BUTTON;

	object_class->finalize = galeon_radio_action_finalize;

	action_class->activate = galeon_radio_action_activate;
}

static void
galeon_radio_action_init(GaleonRadioAction *action)
{
	action->group = g_slist_prepend(NULL, action);
}

static void
galeon_radio_action_finalize(GObject *object)
{
	GaleonRadioAction *action;
	GSList *tmp_list;

	g_return_if_fail(GALEON_IS_RADIO_ACTION(object));

	action = GALEON_RADIO_ACTION(object);

	action->group = g_slist_remove(action->group, action);

	tmp_list = action->group;

	while (tmp_list)
	{
		GaleonRadioAction *tmp_action = tmp_list->data;

		tmp_list = tmp_list->next;
		tmp_action->group = action->group;
	}

	if (parent_class->finalize)
		(* parent_class->finalize) (object);
}

static void
galeon_radio_action_activate(EggAction *action)
{
	GaleonRadioAction *radio_action;
	GaleonToggleAction *toggle_action;
	GaleonToggleAction *tmp_action;
	GSList *tmp_list;

	g_return_if_fail(GALEON_IS_RADIO_ACTION(action));

	radio_action = GALEON_RADIO_ACTION(action);
	toggle_action = GALEON_TOGGLE_ACTION(action);

	if (toggle_action->active)
	{
		tmp_list = radio_action->group;

		while (tmp_list)
		{
			tmp_action = tmp_list->data;
			tmp_list = tmp_list->next;

			if (tmp_action->active && (tmp_action != toggle_action)) 
			{
				toggle_action->active = !toggle_action->active;
				break;
			}
		}
	}
	else
	{
		toggle_action->active = !toggle_action->active;

		tmp_list = radio_action->group;
		while (tmp_list)
		{
			tmp_action = tmp_list->data;
			tmp_list = tmp_list->next;

			if (tmp_action->active && (tmp_action != toggle_action))
			{
				egg_action_activate(EGG_ACTION(tmp_action));
				break;
			}
		}
	}

	galeon_toggle_action_toggled(toggle_action);
}

/**
 * galeon_radio_action_get_group:
 * @action: the action object
 *
 * Returns: the list representing the radio group for this object
 */
GSList *
galeon_radio_action_get_group(GaleonRadioAction *action)
{
	g_return_val_if_fail(GALEON_IS_RADIO_ACTION(action), NULL);

	return action->group;
}

/**
 * galeon_radio_action_set_group:
 * @action: the action object
 * @group: a list representing a radio group
 *
 * Sets the radio group for the radio action object.
 */
void
galeon_radio_action_set_group(GaleonRadioAction *action, GSList *group)
{
	g_return_if_fail(GALEON_IS_RADIO_ACTION (action));
	g_return_if_fail(!g_slist_find (group, action));

	if (action->group)
	{
		GSList *slist;

		action->group = g_slist_remove(action->group, action);

		for (slist = action->group; slist; slist = slist->next)
		{
			GaleonRadioAction *tmp_action = slist->data;

			tmp_action->group = action->group;
		}
	}

	action->group = g_slist_prepend(group, action);

	if (group)
	{
		GSList *slist;

		for (slist = action->group; slist; slist = slist->next)
		{
			GaleonRadioAction *tmp_action = slist->data;

			tmp_action->group = action->group;
		}
	}
	else
	{
		GALEON_TOGGLE_ACTION(action)->active = TRUE;
	}
}
