/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-tearoff-action.h"
#include "eel-gconf-extensions.h"
#include <gtk/gtktearoffmenuitem.h>

static void galeon_tearoff_action_init       (GaleonTearoffAction *action);
static void galeon_tearoff_action_class_init (GaleonTearoffActionClass *class);

static GObjectClass *parent_class = NULL;

GType
galeon_tearoff_action_get_type (void)
{
	static GType type = 0;

	if (!type)
	{
		static const GTypeInfo type_info =
		{
			sizeof (GaleonTearoffActionClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) galeon_tearoff_action_class_init,
			(GClassFinalizeFunc) NULL,
			NULL,
			sizeof (GaleonTearoffAction),
			0, /* n_preallocs */
			(GInstanceInitFunc) galeon_tearoff_action_init,
		};

		type = g_type_register_static (EGG_TYPE_ACTION,
					       "GaleonTearoffAction",
					       &type_info, 0);
	}
	return type;
}

static void
galeon_tearoff_show_cb (GtkWidget *w)
{
	/* hide it again */
	gtk_widget_hide (w);
}

static void
connect_proxy (EggAction *action, GtkWidget *proxy)
{
	gboolean has_tearoff = eel_gconf_get_boolean ("/desktop/gnome/interface/menus_have_tearoff");

	if (!has_tearoff)
	{
		/* hide the widget if no tearoffs wanted */
		/* we can't just hide it because the EggMenuMerge will show it later */
		g_signal_connect (proxy, "show",
				  G_CALLBACK (galeon_tearoff_show_cb), NULL);
		gtk_widget_hide (proxy);
	}

	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
}

static void
galeon_tearoff_action_finalize (GObject *object)
{
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
galeon_tearoff_action_class_init (GaleonTearoffActionClass *class)
{
	EggActionClass *action_class;
	GObjectClass *object_class = G_OBJECT_CLASS (class);

	object_class->finalize = galeon_tearoff_action_finalize;

	parent_class = g_type_class_peek_parent (class);
	action_class = EGG_ACTION_CLASS (class);

	action_class->menu_item_type = GTK_TYPE_TEAROFF_MENU_ITEM;
	action_class->connect_proxy = connect_proxy;

}

static void
galeon_tearoff_action_init (GaleonTearoffAction *action)
{
}


